# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["TranscriptionEngineDeepgramConfigParam"]


class TranscriptionEngineDeepgramConfigParam(TypedDict, total=False):
    transcription_engine: Required[Literal["Deepgram"]]
    """Engine identifier for Deepgram transcription service"""

    transcription_model: Required[Literal["deepgram/nova-2", "deepgram/nova-3"]]
    """The model to use for transcription."""

    language: Literal[
        "bg",
        "ca",
        "zh",
        "zh-CN",
        "zh-Hans",
        "zh-TW",
        "zh-Hant",
        "zh-HK",
        "cs",
        "da",
        "da-DK",
        "nl",
        "en",
        "en-US",
        "en-AU",
        "en-GB",
        "en-NZ",
        "en-IN",
        "et",
        "fi",
        "nl-BE",
        "fr",
        "fr-CA",
        "de",
        "de-CH",
        "el",
        "hi",
        "hu",
        "id",
        "it",
        "ja",
        "ko",
        "ko-KR",
        "lv",
        "lt",
        "ms",
        "no",
        "pl",
        "pt",
        "pt-BR",
        "pt-PT",
        "ro",
        "ru",
        "sk",
        "es",
        "es-419",
        "sv",
        "sv-SE",
        "th",
        "th-TH",
        "tr",
        "uk",
        "vi",
        "auto_detect",
    ]
    """Language to use for speech recognition.

    Available languages depend on the selected model.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["CampaignSuspendedEvent"]


class CampaignSuspendedEvent(BaseModel):
    campaign_id: Optional[str] = FieldInfo(alias="campaignId", default=None)
    """The ID of the campaign."""

    description: Optional[str] = None
    """Description of the event."""

    status: Optional[Literal["DORMANT"]] = None
    """The status of the campaign."""

    type: Optional[Literal["TELNYX_EVENT"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ReportListParams", "Filter", "Page"]


class ReportListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[report_type], filter[status]
    """

    page: Page
    """Consolidated page parameter (deepObject style).

    Originally: page[number], page[size]
    """


class Filter(TypedDict, total=False):
    report_type: Literal["export_portouts_csv"]
    """Filter reports of a specific type"""

    status: Literal["pending", "completed"]
    """Filter reports of a specific status"""


class Page(TypedDict, total=False):
    number: int
    """The page number to load"""

    size: int
    """The size of the page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .sim_card_status import SimCardStatus

__all__ = ["SimpleSimCard", "CurrentBillingPeriodConsumedData", "DataLimit"]


class CurrentBillingPeriodConsumedData(BaseModel):
    amount: Optional[str] = None

    unit: Optional[str] = None


class DataLimit(BaseModel):
    amount: Optional[str] = None

    unit: Optional[Literal["MB", "GB"]] = None


class SimpleSimCard(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    actions_in_progress: Optional[bool] = None
    """Indicate whether the SIM card has any pending (in-progress) actions."""

    authorized_imeis: Optional[List[str]] = None
    """List of IMEIs authorized to use a given SIM card."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    current_billing_period_consumed_data: Optional[CurrentBillingPeriodConsumedData] = None
    """The SIM card consumption so far in the current billing cycle."""

    data_limit: Optional[DataLimit] = None
    """The SIM card individual data limit configuration."""

    eid: Optional[str] = None
    """The Embedded Identity Document (eID) for eSIM cards."""

    esim_installation_status: Optional[Literal["released", "disabled"]] = None
    """The installation status of the eSIM. Only applicable for eSIM cards."""

    iccid: Optional[str] = None
    """The ICCID is the identifier of the specific SIM card/chip.

    Each SIM is internationally identified by its integrated circuit card identifier
    (ICCID). ICCIDs are stored in the SIM card's memory and are also engraved or
    printed on the SIM card body during a process called personalization.
    """

    imsi: Optional[str] = None
    """
    SIM cards are identified on their individual network operators by a unique
    International Mobile Subscriber Identity (IMSI). <br/> Mobile network operators
    connect mobile phone calls and communicate with their market SIM cards using
    their IMSIs. The IMSI is stored in the Subscriber Identity Module (SIM) inside
    the device and is sent by the device to the appropriate network. It is used to
    acquire the details of the device in the Home Location Register (HLR) or the
    Visitor Location Register (VLR).
    """

    msisdn: Optional[str] = None
    """
    Mobile Station International Subscriber Directory Number (MSISDN) is a number
    used to identify a mobile phone number internationally. <br/> MSISDN is defined
    by the E.164 numbering plan. It includes a country code and a National
    Destination Code which identifies the subscriber's operator.
    """

    record_type: Optional[str] = None

    resources_with_in_progress_actions: Optional[List[Dict[str, object]]] = None
    """List of resources with actions in progress."""

    sim_card_group_id: Optional[str] = None
    """The group SIMCardGroup identification.

    This attribute can be <code>null</code> when it's present in an associated
    resource.
    """

    status: Optional[SimCardStatus] = None

    tags: Optional[List[str]] = None
    """Searchable tags associated with the SIM card"""

    type: Optional[Literal["physical", "esim"]] = None
    """The type of SIM card"""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was updated."""

    version: Optional[str] = None
    """The version of the SIM card."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import room_recording_list_params, room_recording_delete_bulk_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultPagination, AsyncDefaultPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.room_recording_list_response import RoomRecordingListResponse
from ..types.room_recording_retrieve_response import RoomRecordingRetrieveResponse
from ..types.room_recording_delete_bulk_response import RoomRecordingDeleteBulkResponse

__all__ = ["RoomRecordingsResource", "AsyncRoomRecordingsResource"]


class RoomRecordingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RoomRecordingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RoomRecordingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RoomRecordingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RoomRecordingsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        room_recording_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomRecordingRetrieveResponse:
        """
        View a room recording.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_recording_id:
            raise ValueError(f"Expected a non-empty value for `room_recording_id` but received {room_recording_id!r}")
        return self._get(
            f"/room_recordings/{room_recording_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoomRecordingRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: room_recording_list_params.Filter | Omit = omit,
        page: room_recording_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultPagination[RoomRecordingListResponse]:
        """
        View a list of room recordings.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_ended_at][eq], filter[date_ended_at][gte],
              filter[date_ended_at][lte], filter[date_started_at][eq],
              filter[date_started_at][gte], filter[date_started_at][lte], filter[room_id],
              filter[participant_id], filter[session_id], filter[status], filter[type],
              filter[duration_secs]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/room_recordings",
            page=SyncDefaultPagination[RoomRecordingListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    room_recording_list_params.RoomRecordingListParams,
                ),
            ),
            model=RoomRecordingListResponse,
        )

    def delete(
        self,
        room_recording_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Synchronously delete a Room Recording.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_recording_id:
            raise ValueError(f"Expected a non-empty value for `room_recording_id` but received {room_recording_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/room_recordings/{room_recording_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def delete_bulk(
        self,
        *,
        filter: room_recording_delete_bulk_params.Filter | Omit = omit,
        page: room_recording_delete_bulk_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomRecordingDeleteBulkResponse:
        """
        Delete several room recordings in a bulk.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_ended_at][eq], filter[date_ended_at][gte],
              filter[date_ended_at][lte], filter[date_started_at][eq],
              filter[date_started_at][gte], filter[date_started_at][lte], filter[room_id],
              filter[participant_id], filter[session_id], filter[status], filter[type],
              filter[duration_secs]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._delete(
            "/room_recordings",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    room_recording_delete_bulk_params.RoomRecordingDeleteBulkParams,
                ),
            ),
            cast_to=RoomRecordingDeleteBulkResponse,
        )


class AsyncRoomRecordingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRoomRecordingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRoomRecordingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRoomRecordingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRoomRecordingsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        room_recording_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomRecordingRetrieveResponse:
        """
        View a room recording.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_recording_id:
            raise ValueError(f"Expected a non-empty value for `room_recording_id` but received {room_recording_id!r}")
        return await self._get(
            f"/room_recordings/{room_recording_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoomRecordingRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: room_recording_list_params.Filter | Omit = omit,
        page: room_recording_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RoomRecordingListResponse, AsyncDefaultPagination[RoomRecordingListResponse]]:
        """
        View a list of room recordings.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_ended_at][eq], filter[date_ended_at][gte],
              filter[date_ended_at][lte], filter[date_started_at][eq],
              filter[date_started_at][gte], filter[date_started_at][lte], filter[room_id],
              filter[participant_id], filter[session_id], filter[status], filter[type],
              filter[duration_secs]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/room_recordings",
            page=AsyncDefaultPagination[RoomRecordingListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    room_recording_list_params.RoomRecordingListParams,
                ),
            ),
            model=RoomRecordingListResponse,
        )

    async def delete(
        self,
        room_recording_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Synchronously delete a Room Recording.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_recording_id:
            raise ValueError(f"Expected a non-empty value for `room_recording_id` but received {room_recording_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/room_recordings/{room_recording_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def delete_bulk(
        self,
        *,
        filter: room_recording_delete_bulk_params.Filter | Omit = omit,
        page: room_recording_delete_bulk_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RoomRecordingDeleteBulkResponse:
        """
        Delete several room recordings in a bulk.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[date_ended_at][eq], filter[date_ended_at][gte],
              filter[date_ended_at][lte], filter[date_started_at][eq],
              filter[date_started_at][gte], filter[date_started_at][lte], filter[room_id],
              filter[participant_id], filter[session_id], filter[status], filter[type],
              filter[duration_secs]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._delete(
            "/room_recordings",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                    },
                    room_recording_delete_bulk_params.RoomRecordingDeleteBulkParams,
                ),
            ),
            cast_to=RoomRecordingDeleteBulkResponse,
        )


class RoomRecordingsResourceWithRawResponse:
    def __init__(self, room_recordings: RoomRecordingsResource) -> None:
        self._room_recordings = room_recordings

        self.retrieve = to_raw_response_wrapper(
            room_recordings.retrieve,
        )
        self.list = to_raw_response_wrapper(
            room_recordings.list,
        )
        self.delete = to_raw_response_wrapper(
            room_recordings.delete,
        )
        self.delete_bulk = to_raw_response_wrapper(
            room_recordings.delete_bulk,
        )


class AsyncRoomRecordingsResourceWithRawResponse:
    def __init__(self, room_recordings: AsyncRoomRecordingsResource) -> None:
        self._room_recordings = room_recordings

        self.retrieve = async_to_raw_response_wrapper(
            room_recordings.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            room_recordings.list,
        )
        self.delete = async_to_raw_response_wrapper(
            room_recordings.delete,
        )
        self.delete_bulk = async_to_raw_response_wrapper(
            room_recordings.delete_bulk,
        )


class RoomRecordingsResourceWithStreamingResponse:
    def __init__(self, room_recordings: RoomRecordingsResource) -> None:
        self._room_recordings = room_recordings

        self.retrieve = to_streamed_response_wrapper(
            room_recordings.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            room_recordings.list,
        )
        self.delete = to_streamed_response_wrapper(
            room_recordings.delete,
        )
        self.delete_bulk = to_streamed_response_wrapper(
            room_recordings.delete_bulk,
        )


class AsyncRoomRecordingsResourceWithStreamingResponse:
    def __init__(self, room_recordings: AsyncRoomRecordingsResource) -> None:
        self._room_recordings = room_recordings

        self.retrieve = async_to_streamed_response_wrapper(
            room_recordings.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            room_recordings.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            room_recordings.delete,
        )
        self.delete_bulk = async_to_streamed_response_wrapper(
            room_recordings.delete_bulk,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from typing_extensions import TypeAlias

__all__ = ["EnumerationRetrieveResponse"]

EnumerationRetrieveResponse: TypeAlias = Union[List[str], object, object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, Annotated, TypeAlias, TypedDict

from .._types import Base64FileInput
from .._utils import PropertyInfo
from .._models import set_pydantic_config

__all__ = [
    "DocumentUploadJsonParams",
    "Document",
    "DocumentDocServiceDocumentUploadURL",
    "DocumentDocServiceDocumentUploadInline",
]


class DocumentUploadJsonParams(TypedDict, total=False):
    document: Required[Document]


class DocumentDocServiceDocumentUploadURL(TypedDict, total=False):
    url: Required[str]
    """
    If the file is already hosted publicly, you can provide a URL and have the
    documents service fetch it for you.
    """

    customer_reference: str
    """Optional reference string for customer tracking."""

    filename: str
    """The filename of the document."""


class DocumentDocServiceDocumentUploadInline(TypedDict, total=False):
    file: Required[Annotated[Union[str, Base64FileInput], PropertyInfo(format="base64")]]
    """The Base64 encoded contents of the file you are uploading."""

    customer_reference: str
    """A customer reference string for customer look ups."""

    filename: str
    """The filename of the document."""


set_pydantic_config(DocumentDocServiceDocumentUploadInline, {"arbitrary_types_allowed": True})

Document: TypeAlias = Union[DocumentDocServiceDocumentUploadURL, DocumentDocServiceDocumentUploadInline]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultPagination, AsyncDefaultPagination
from telnyx.types.bundle_pricing import (
    UserBundle,
    UserBundleCreateResponse,
    UserBundleRetrieveResponse,
    UserBundleDeactivateResponse,
    UserBundleListUnusedResponse,
    UserBundleListResourcesResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUserBundles:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        user_bundle = client.bundle_pricing.user_bundles.create()
        assert_matches_type(UserBundleCreateResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        user_bundle = client.bundle_pricing.user_bundles.create(
            idempotency_key="12ade33a-21c0-473b-b055-b3c836e1c292",
            items=[
                {
                    "billing_bundle_id": "12ade33a-21c0-473b-b055-b3c836e1c292",
                    "quantity": 0,
                }
            ],
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(UserBundleCreateResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.bundle_pricing.user_bundles.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_bundle = response.parse()
        assert_matches_type(UserBundleCreateResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.bundle_pricing.user_bundles.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_bundle = response.parse()
            assert_matches_type(UserBundleCreateResponse, user_bundle, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        user_bundle = client.bundle_pricing.user_bundles.retrieve(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        )
        assert_matches_type(UserBundleRetrieveResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Telnyx) -> None:
        user_bundle = client.bundle_pricing.user_bundles.retrieve(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(UserBundleRetrieveResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.bundle_pricing.user_bundles.with_raw_response.retrieve(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_bundle = response.parse()
        assert_matches_type(UserBundleRetrieveResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.bundle_pricing.user_bundles.with_streaming_response.retrieve(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_bundle = response.parse()
            assert_matches_type(UserBundleRetrieveResponse, user_bundle, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_bundle_id` but received ''"):
            client.bundle_pricing.user_bundles.with_raw_response.retrieve(
                user_bundle_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        user_bundle = client.bundle_pricing.user_bundles.list()
        assert_matches_type(SyncDefaultPagination[UserBundle], user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        user_bundle = client.bundle_pricing.user_bundles.list(
            filter={
                "country_iso": ["US"],
                "resource": ["+15617819942"],
            },
            page={
                "number": 1,
                "size": 1,
            },
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(SyncDefaultPagination[UserBundle], user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.bundle_pricing.user_bundles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_bundle = response.parse()
        assert_matches_type(SyncDefaultPagination[UserBundle], user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.bundle_pricing.user_bundles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_bundle = response.parse()
            assert_matches_type(SyncDefaultPagination[UserBundle], user_bundle, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_deactivate(self, client: Telnyx) -> None:
        user_bundle = client.bundle_pricing.user_bundles.deactivate(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        )
        assert_matches_type(UserBundleDeactivateResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_deactivate_with_all_params(self, client: Telnyx) -> None:
        user_bundle = client.bundle_pricing.user_bundles.deactivate(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(UserBundleDeactivateResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_deactivate(self, client: Telnyx) -> None:
        response = client.bundle_pricing.user_bundles.with_raw_response.deactivate(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_bundle = response.parse()
        assert_matches_type(UserBundleDeactivateResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_deactivate(self, client: Telnyx) -> None:
        with client.bundle_pricing.user_bundles.with_streaming_response.deactivate(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_bundle = response.parse()
            assert_matches_type(UserBundleDeactivateResponse, user_bundle, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_deactivate(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_bundle_id` but received ''"):
            client.bundle_pricing.user_bundles.with_raw_response.deactivate(
                user_bundle_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_resources(self, client: Telnyx) -> None:
        user_bundle = client.bundle_pricing.user_bundles.list_resources(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        )
        assert_matches_type(UserBundleListResourcesResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_resources_with_all_params(self, client: Telnyx) -> None:
        user_bundle = client.bundle_pricing.user_bundles.list_resources(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(UserBundleListResourcesResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_resources(self, client: Telnyx) -> None:
        response = client.bundle_pricing.user_bundles.with_raw_response.list_resources(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_bundle = response.parse()
        assert_matches_type(UserBundleListResourcesResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_resources(self, client: Telnyx) -> None:
        with client.bundle_pricing.user_bundles.with_streaming_response.list_resources(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_bundle = response.parse()
            assert_matches_type(UserBundleListResourcesResponse, user_bundle, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_resources(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_bundle_id` but received ''"):
            client.bundle_pricing.user_bundles.with_raw_response.list_resources(
                user_bundle_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_unused(self, client: Telnyx) -> None:
        user_bundle = client.bundle_pricing.user_bundles.list_unused()
        assert_matches_type(UserBundleListUnusedResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_unused_with_all_params(self, client: Telnyx) -> None:
        user_bundle = client.bundle_pricing.user_bundles.list_unused(
            filter={
                "country_iso": ["US"],
                "resource": ["+15617819942"],
            },
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(UserBundleListUnusedResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_unused(self, client: Telnyx) -> None:
        response = client.bundle_pricing.user_bundles.with_raw_response.list_unused()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_bundle = response.parse()
        assert_matches_type(UserBundleListUnusedResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_unused(self, client: Telnyx) -> None:
        with client.bundle_pricing.user_bundles.with_streaming_response.list_unused() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_bundle = response.parse()
            assert_matches_type(UserBundleListUnusedResponse, user_bundle, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncUserBundles:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        user_bundle = await async_client.bundle_pricing.user_bundles.create()
        assert_matches_type(UserBundleCreateResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        user_bundle = await async_client.bundle_pricing.user_bundles.create(
            idempotency_key="12ade33a-21c0-473b-b055-b3c836e1c292",
            items=[
                {
                    "billing_bundle_id": "12ade33a-21c0-473b-b055-b3c836e1c292",
                    "quantity": 0,
                }
            ],
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(UserBundleCreateResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.bundle_pricing.user_bundles.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_bundle = await response.parse()
        assert_matches_type(UserBundleCreateResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.bundle_pricing.user_bundles.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_bundle = await response.parse()
            assert_matches_type(UserBundleCreateResponse, user_bundle, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        user_bundle = await async_client.bundle_pricing.user_bundles.retrieve(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        )
        assert_matches_type(UserBundleRetrieveResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncTelnyx) -> None:
        user_bundle = await async_client.bundle_pricing.user_bundles.retrieve(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(UserBundleRetrieveResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.bundle_pricing.user_bundles.with_raw_response.retrieve(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_bundle = await response.parse()
        assert_matches_type(UserBundleRetrieveResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.bundle_pricing.user_bundles.with_streaming_response.retrieve(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_bundle = await response.parse()
            assert_matches_type(UserBundleRetrieveResponse, user_bundle, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_bundle_id` but received ''"):
            await async_client.bundle_pricing.user_bundles.with_raw_response.retrieve(
                user_bundle_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        user_bundle = await async_client.bundle_pricing.user_bundles.list()
        assert_matches_type(AsyncDefaultPagination[UserBundle], user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        user_bundle = await async_client.bundle_pricing.user_bundles.list(
            filter={
                "country_iso": ["US"],
                "resource": ["+15617819942"],
            },
            page={
                "number": 1,
                "size": 1,
            },
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(AsyncDefaultPagination[UserBundle], user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.bundle_pricing.user_bundles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_bundle = await response.parse()
        assert_matches_type(AsyncDefaultPagination[UserBundle], user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.bundle_pricing.user_bundles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_bundle = await response.parse()
            assert_matches_type(AsyncDefaultPagination[UserBundle], user_bundle, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_deactivate(self, async_client: AsyncTelnyx) -> None:
        user_bundle = await async_client.bundle_pricing.user_bundles.deactivate(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        )
        assert_matches_type(UserBundleDeactivateResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_deactivate_with_all_params(self, async_client: AsyncTelnyx) -> None:
        user_bundle = await async_client.bundle_pricing.user_bundles.deactivate(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(UserBundleDeactivateResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_deactivate(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.bundle_pricing.user_bundles.with_raw_response.deactivate(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_bundle = await response.parse()
        assert_matches_type(UserBundleDeactivateResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_deactivate(self, async_client: AsyncTelnyx) -> None:
        async with async_client.bundle_pricing.user_bundles.with_streaming_response.deactivate(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_bundle = await response.parse()
            assert_matches_type(UserBundleDeactivateResponse, user_bundle, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_deactivate(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_bundle_id` but received ''"):
            await async_client.bundle_pricing.user_bundles.with_raw_response.deactivate(
                user_bundle_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_resources(self, async_client: AsyncTelnyx) -> None:
        user_bundle = await async_client.bundle_pricing.user_bundles.list_resources(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        )
        assert_matches_type(UserBundleListResourcesResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_resources_with_all_params(self, async_client: AsyncTelnyx) -> None:
        user_bundle = await async_client.bundle_pricing.user_bundles.list_resources(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(UserBundleListResourcesResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_resources(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.bundle_pricing.user_bundles.with_raw_response.list_resources(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_bundle = await response.parse()
        assert_matches_type(UserBundleListResourcesResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_resources(self, async_client: AsyncTelnyx) -> None:
        async with async_client.bundle_pricing.user_bundles.with_streaming_response.list_resources(
            user_bundle_id="ca1d2263-d1f1-43ac-ba53-248e7a4bb26a",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_bundle = await response.parse()
            assert_matches_type(UserBundleListResourcesResponse, user_bundle, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_resources(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_bundle_id` but received ''"):
            await async_client.bundle_pricing.user_bundles.with_raw_response.list_resources(
                user_bundle_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_unused(self, async_client: AsyncTelnyx) -> None:
        user_bundle = await async_client.bundle_pricing.user_bundles.list_unused()
        assert_matches_type(UserBundleListUnusedResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_unused_with_all_params(self, async_client: AsyncTelnyx) -> None:
        user_bundle = await async_client.bundle_pricing.user_bundles.list_unused(
            filter={
                "country_iso": ["US"],
                "resource": ["+15617819942"],
            },
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(UserBundleListUnusedResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_unused(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.bundle_pricing.user_bundles.with_raw_response.list_unused()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user_bundle = await response.parse()
        assert_matches_type(UserBundleListUnusedResponse, user_bundle, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_unused(self, async_client: AsyncTelnyx) -> None:
        async with async_client.bundle_pricing.user_bundles.with_streaming_response.list_unused() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user_bundle = await response.parse()
            assert_matches_type(UserBundleListUnusedResponse, user_bundle, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .record import Record
from .._models import BaseModel
from .network_interface import NetworkInterface

__all__ = ["PublicInternetGatewayListResponse", "PublicInternetGatewayListResponseRegion"]


class PublicInternetGatewayListResponseRegion(BaseModel):
    code: Optional[str] = None
    """Region code of the interface."""

    name: Optional[str] = None
    """Region name of the interface."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""


class PublicInternetGatewayListResponse(Record, NetworkInterface):
    public_ip: Optional[str] = None
    """The publically accessible ip for this interface."""

    region: Optional[PublicInternetGatewayListResponseRegion] = None

    region_code: Optional[str] = None
    """The region interface is deployed to."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from .campaign_suspended_event import CampaignSuspendedEvent
from .campaign_status_update_event import CampaignStatusUpdateEvent

__all__ = ["CampaignStatusUpdateWebhookEvent"]

CampaignStatusUpdateWebhookEvent: TypeAlias = Union[CampaignStatusUpdateEvent, CampaignSuspendedEvent]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx._utils import parse_datetime
from telnyx.types.legacy.reporting.batch_detail_records import (
    VoiceListResponse,
    VoiceCreateResponse,
    VoiceDeleteResponse,
    VoiceRetrieveResponse,
    VoiceRetrieveFieldsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVoice:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        voice = client.legacy.reporting.batch_detail_records.voice.create(
            end_time=parse_datetime("2024-02-12T23:59:59Z"),
            start_time=parse_datetime("2024-02-01T00:00:00Z"),
        )
        assert_matches_type(VoiceCreateResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        voice = client.legacy.reporting.batch_detail_records.voice.create(
            end_time=parse_datetime("2024-02-12T23:59:59Z"),
            start_time=parse_datetime("2024-02-01T00:00:00Z"),
            call_types=[1, 2],
            connections=[123, 456],
            fields=["call_leg_id", "start_time", "end_time"],
            filters=[
                {
                    "billing_group": "adfaa016-f921-4b6c-97bb-e4c1dad231c5",
                    "cld": "+13129457420",
                    "cld_filter": "contains",
                    "cli": "+13129457420",
                    "cli_filter": "contains",
                    "filter_type": "and",
                    "tags_list": "tag1",
                }
            ],
            include_all_metadata=True,
            managed_accounts=["f47ac10b-58cc-4372-a567-0e02b2c3d479", "6ba7b810-9dad-11d1-80b4-00c04fd430c8"],
            record_types=[1, 2],
            report_name="My CDR Report",
            select_all_managed_accounts=False,
            source="calls",
            timezone="UTC",
        )
        assert_matches_type(VoiceCreateResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.legacy.reporting.batch_detail_records.voice.with_raw_response.create(
            end_time=parse_datetime("2024-02-12T23:59:59Z"),
            start_time=parse_datetime("2024-02-01T00:00:00Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = response.parse()
        assert_matches_type(VoiceCreateResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.legacy.reporting.batch_detail_records.voice.with_streaming_response.create(
            end_time=parse_datetime("2024-02-12T23:59:59Z"),
            start_time=parse_datetime("2024-02-01T00:00:00Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = response.parse()
            assert_matches_type(VoiceCreateResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        voice = client.legacy.reporting.batch_detail_records.voice.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(VoiceRetrieveResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.legacy.reporting.batch_detail_records.voice.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = response.parse()
        assert_matches_type(VoiceRetrieveResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.legacy.reporting.batch_detail_records.voice.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = response.parse()
            assert_matches_type(VoiceRetrieveResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.legacy.reporting.batch_detail_records.voice.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        voice = client.legacy.reporting.batch_detail_records.voice.list()
        assert_matches_type(VoiceListResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.legacy.reporting.batch_detail_records.voice.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = response.parse()
        assert_matches_type(VoiceListResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.legacy.reporting.batch_detail_records.voice.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = response.parse()
            assert_matches_type(VoiceListResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        voice = client.legacy.reporting.batch_detail_records.voice.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(VoiceDeleteResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.legacy.reporting.batch_detail_records.voice.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = response.parse()
        assert_matches_type(VoiceDeleteResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.legacy.reporting.batch_detail_records.voice.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = response.parse()
            assert_matches_type(VoiceDeleteResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.legacy.reporting.batch_detail_records.voice.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_fields(self, client: Telnyx) -> None:
        voice = client.legacy.reporting.batch_detail_records.voice.retrieve_fields()
        assert_matches_type(VoiceRetrieveFieldsResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_fields(self, client: Telnyx) -> None:
        response = client.legacy.reporting.batch_detail_records.voice.with_raw_response.retrieve_fields()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = response.parse()
        assert_matches_type(VoiceRetrieveFieldsResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_fields(self, client: Telnyx) -> None:
        with client.legacy.reporting.batch_detail_records.voice.with_streaming_response.retrieve_fields() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = response.parse()
            assert_matches_type(VoiceRetrieveFieldsResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncVoice:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        voice = await async_client.legacy.reporting.batch_detail_records.voice.create(
            end_time=parse_datetime("2024-02-12T23:59:59Z"),
            start_time=parse_datetime("2024-02-01T00:00:00Z"),
        )
        assert_matches_type(VoiceCreateResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        voice = await async_client.legacy.reporting.batch_detail_records.voice.create(
            end_time=parse_datetime("2024-02-12T23:59:59Z"),
            start_time=parse_datetime("2024-02-01T00:00:00Z"),
            call_types=[1, 2],
            connections=[123, 456],
            fields=["call_leg_id", "start_time", "end_time"],
            filters=[
                {
                    "billing_group": "adfaa016-f921-4b6c-97bb-e4c1dad231c5",
                    "cld": "+13129457420",
                    "cld_filter": "contains",
                    "cli": "+13129457420",
                    "cli_filter": "contains",
                    "filter_type": "and",
                    "tags_list": "tag1",
                }
            ],
            include_all_metadata=True,
            managed_accounts=["f47ac10b-58cc-4372-a567-0e02b2c3d479", "6ba7b810-9dad-11d1-80b4-00c04fd430c8"],
            record_types=[1, 2],
            report_name="My CDR Report",
            select_all_managed_accounts=False,
            source="calls",
            timezone="UTC",
        )
        assert_matches_type(VoiceCreateResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.legacy.reporting.batch_detail_records.voice.with_raw_response.create(
            end_time=parse_datetime("2024-02-12T23:59:59Z"),
            start_time=parse_datetime("2024-02-01T00:00:00Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = await response.parse()
        assert_matches_type(VoiceCreateResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.legacy.reporting.batch_detail_records.voice.with_streaming_response.create(
            end_time=parse_datetime("2024-02-12T23:59:59Z"),
            start_time=parse_datetime("2024-02-01T00:00:00Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = await response.parse()
            assert_matches_type(VoiceCreateResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        voice = await async_client.legacy.reporting.batch_detail_records.voice.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(VoiceRetrieveResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.legacy.reporting.batch_detail_records.voice.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = await response.parse()
        assert_matches_type(VoiceRetrieveResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.legacy.reporting.batch_detail_records.voice.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = await response.parse()
            assert_matches_type(VoiceRetrieveResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.legacy.reporting.batch_detail_records.voice.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        voice = await async_client.legacy.reporting.batch_detail_records.voice.list()
        assert_matches_type(VoiceListResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.legacy.reporting.batch_detail_records.voice.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = await response.parse()
        assert_matches_type(VoiceListResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.legacy.reporting.batch_detail_records.voice.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = await response.parse()
            assert_matches_type(VoiceListResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        voice = await async_client.legacy.reporting.batch_detail_records.voice.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(VoiceDeleteResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.legacy.reporting.batch_detail_records.voice.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = await response.parse()
        assert_matches_type(VoiceDeleteResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.legacy.reporting.batch_detail_records.voice.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = await response.parse()
            assert_matches_type(VoiceDeleteResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.legacy.reporting.batch_detail_records.voice.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_fields(self, async_client: AsyncTelnyx) -> None:
        voice = await async_client.legacy.reporting.batch_detail_records.voice.retrieve_fields()
        assert_matches_type(VoiceRetrieveFieldsResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_fields(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.legacy.reporting.batch_detail_records.voice.with_raw_response.retrieve_fields()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        voice = await response.parse()
        assert_matches_type(VoiceRetrieveFieldsResponse, voice, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_fields(self, async_client: AsyncTelnyx) -> None:
        async with (
            async_client.legacy.reporting.batch_detail_records.voice.with_streaming_response.retrieve_fields()
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            voice = await response.parse()
            assert_matches_type(VoiceRetrieveFieldsResponse, voice, path=["response"])

        assert cast(Any, response.is_closed) is True
